#!/usr/bin/env python3
# -*- coding: utf-8 -*-


from jira import JIRA
import sys
import os
import ast

sys.path.append(os.path.dirname(os.path.realpath(__file__)))  # noqa
import config
import des

jira_issues_api = 'https://jira.cvte.com/issues/?jql='

jira_filter = 'assignee = currentUser() AND type not in (新需求) AND resolution = Unresolved order by updated DESC',

issues_config_key = 'issues'
username_config_key = 'username'
password_config_key = 'password'


def GetJiraIssuesFromServer():
    username_en = config.Get(username_config_key)
    password_en = config.Get(password_config_key)
    if not username_en or not password_en:
        print("请通过'imit config' 配置Jenkines用户名和密码!")
        return []

    username = des.DesDecrypt(username_en)
    password = des.DesDecrypt(password_en)
    jira = JIRA(server='https://jira.cvte.com/',
                basic_auth=(username, password),
                timeout=5)

    issues_list = [(issues.key, issues.fields.summary)
                   for issues in jira.search_issues(jira_filter)]
    config.Write(issues_config_key, str(issues_list).strip('[]'))
    return issues_list


def GetJiraIssuesFromConfig():
    issues_str = config.Get(issues_config_key)
    if not issues_str:
        return []

    issues_list = list(ast.literal_eval(issues_str))
    return issues_list


def GetJiraIssues():
    issues_list = GetJiraIssuesFromConfig()
    if not issues_list:
        issues_list = GetJiraIssuesFromServer()

    return issues_list



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import os
import sys


sys.path.append(os.path.dirname(os.path.realpath(__file__)))  # noqa
import version_handler
import commit_inquirer


def _GetCommitType(args, commit_types):
    for i in commit_types:
        if args[i]:
            return commit_types[i]
    return ''


def _GetVersionIndex(args):
    for i in range(1, 5):
        key = '-%d' % i
        if args[key]:
            return i
    return 0


def _GetCommitMsg(args):
    return args['-M']


def UpdateOptionFromArgs(commit_option, args, commit_types):
    commit_type = _GetCommitType(args, commit_types)
    if commit_type and 'commit_type' not in commit_option:
        commit_option['commit_type'] = commit_type

    version_index = _GetVersionIndex(args)
    if version_index > 0 and 'version_index' not in commit_option:
        commit_option['version_index'] = version_index

    commit_msg = _GetCommitMsg(args)
    if commit_msg and 'commit_msg' not in commit_option:
        commit_option['commit_msg'] = commit_msg

    logging.debug('option from args: ' + str(commit_option))


def UpdateOptionFromInquirer(commit_option, version_file_path, commit_types):
    version_dict = version_handler.TagNumDict(version_file_path)
    logging.debug('tag num dict: ' + str(version_dict))

    if 'commit_type' not in commit_option:
        answer = commit_inquirer.QType(list(commit_types.values()))
        commit_option.update(answer)
        logging.debug('option : '+str(answer))

    if 'version_index' not in commit_option:
        answer = commit_inquirer.QVersion(
            version_dict.keys(), commit_option['commit_type'], version_file_path)
        commit_option.update(answer)
        logging.debug('option : '+str(answer))

    if 'jira_id' not in commit_option:
        answer = commit_inquirer.QJiraId()
        commit_option.update(answer)
        logging.debug('option : '+str(answer))

    if 'commit_msg' not in commit_option:
        answer = commit_inquirer.QMsg()
        commit_option.update(answer)
        logging.debug('option : '+str(answer))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging


def TagNumDict(version_file_path):
    tag_num_dict = {}
    with open(version_file_path, 'r') as file:
        for line in file.readlines():
            line = line.strip()
            tag = line.split('=')[0]
            num = line.split('=')[1]

            if not num.isdigit():
                raise ValueError("version number must be digit!")
            tag_num_dict[tag] = int(num)
    return tag_num_dict


def CurrentVersion(version_file_path):
    nums_list = list(TagNumDict(version_file_path).values())
    return _VersionStr(nums_list)


def _IncreaseVersion(nums_list, index):
    if not nums_list or index <= 0:
        raise Exception('nums list is empty or index < 0')

    index = index - 1
    if index >= len(nums_list):
        index = len(nums_list) - 1

    nums_list[index] += 1
    for i in range(index+1, len(nums_list)):
        nums_list[i] = 0


def _ToFile(version_dict, version_file_path):
    with open(version_file_path, 'w') as file:
        for key in version_dict:
            line = '%s=%s\n' % (key, version_dict[key])
            file.write(line)


def _VersionStr(num_list):
    version = ''
    for num in num_list:
        version += '%s.' % num

    if not version:
        raise ValueError("version string is empty")

    return version[:-1]


def Handle(version_index, version_file_path):
    tag_num_dict = TagNumDict(version_file_path)
    version_nums = list(tag_num_dict.values())
    if version_index <= 0:
        return _VersionStr(version_nums)

    logging.debug('before change: ' + str(version_nums))
    _IncreaseVersion(version_nums, version_index)
    logging.debug('after change: ' + str(version_nums))

    version_dict = dict(zip(tag_num_dict.keys(), version_nums))
    _ToFile(version_dict, version_file_path)

    return _VersionStr(version_nums)


if __name__ == "__main__":
    print(CurrentVersion('version.properties'))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Usage:
    imit.py [-m|-b|-f|-r] [-1|-2|-3|-4] [-M <msg>] [-h|--help] [--log_level=<log_level>]
    imit.py config

Options:
    -h,--help                   show usage
    -m                          set commit type: modify
    -b                          set commit type: bugfix
    -f                          set commit type: feature
    -r                          set commit type: revert
    -1                          1st version number +1
    -2                          2nd version number +1
    -3                          3rd version number +1
    -4                          4th version number +1
    -M <msg>                    set commit message
    --log_level=<log_level>     set log level：notset,debug,info,warn,error,fatal,critical
"""


import sys
import os
import inquirer
import docopt
import logging
from pprint import pprint

sys.path.append(os.path.dirname(os.path.realpath(__file__)))  # noqa
import commit_inquirer
import option_handler
import git_handler
import config
import des


def InitLogger(args):
    log_level = args["--log_level"]
    if log_level is None:
        log_level = "info"

    logging.basicConfig(level=logging._nameToLevel[log_level.upper()],
                        format='[%(levelname)s]: %(message)s')


def GetCommitOption(args, commit_types, version_file_path):
    commit_option = {}
    # 如果版本文件已修改过，则不再修改版本号(index设为0)
    if version_file_path in git_handler.AllChangedFiles():
        commit_option['version_index'] = 0

    option_handler.UpdateOptionFromArgs(commit_option, args, commit_types)
    option_handler.UpdateOptionFromInquirer(
        commit_option, version_file_path, commit_types)

    return commit_option


def EnvCheck(version_file_path):
    if sys.version_info < (3, 7):  # 3.7后dict默认为有序
        print('Please run imit with greater than python3.7.')
        exit(-1)

    if not os.path.exists('.git') or not os.listdir('.git'):
        print('Please run imit in the git project root path.')
        exit(-1)

    if not os.path.exists(version_file_path):
        print('File %s not found in current path.' % version_file_path)
        exit(-1)

    if not git_handler.StagedFiles():
        print('No staged file, exit commit.')
        exit(-1)


def Config():
    answer = commit_inquirer.QUsernamePassword()
    username = answer['username']
    password = answer['password']
    config.Write('username', des.DesEncrypt(username))
    config.Write('password', des.DesEncrypt(password))


def main():
    commit_types = {'-m': 'modify',
                    '-b': 'bugfix',
                    '-f': 'feature',
                    '-r': 'revert'}
    version_file_path = 'version.properties'

    EnvCheck(version_file_path)

    args = docopt.docopt(__doc__)
    InitLogger(args)
    logging.debug(args)

    if args['config']:
        Config()
        print('Username and password are encrypted and stored in file ~/.imitrc.ini')
        exit(0)

    commit_option = GetCommitOption(args, commit_types, version_file_path)
    logging.debug('commit option: ' + str(commit_option))

    git_handler.Handle(commit_option, version_file_path)


if __name__ == "__main__":
    main()



from pyDes import des, ECB, PAD_PKCS5
import base64

secret_key = ':M)g>0C9'


def DesEncrypt(s):
    iv = secret_key
    k = des(secret_key, ECB, iv, pad=None, padmode=PAD_PKCS5)
    en = k.encrypt(s.encode('utf-8'), padmode=PAD_PKCS5)
    return str(base64.b64encode(en), 'utf-8')


def DesDecrypt(s):
    iv = secret_key
    k = des(secret_key, ECB, iv, pad=None, padmode=PAD_PKCS5)
    de = k.decrypt(base64.b64decode(s), padmode=PAD_PKCS5)
    return str(de, 'utf-8')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import os
import sys
import logging
from git.cmd import Git
from git import Repo

sys.path.append(os.path.dirname(os.path.realpath(__file__)))  # noqa
import version_handler


def _GitCmd(cmd):
    git = Git(os.getcwd())
    return git.execute(cmd)


def _Add(path):
    cmd = ['git', 'add', path]
    _GitCmd(cmd)


def _Commit(msg):
    cmd = ['git', 'commit', '-m', msg]
    _GitCmd(cmd)


def _GenerateCommitMsg(msg_tags, msg):
    commit_msg = ''
    for i in msg_tags:
        if i:
            tag = '[%s]' % i
            commit_msg += tag

    commit_msg += ' ' + msg
    return commit_msg


def _ChangedFiles(cmd):
    output = _GitCmd(cmd)
    logging.debug('changed files: ' + str(output))
    if not output:
        return []
    changed_files = output.split("\n")
    return changed_files


def AllChangedFiles():
    cmd = ['git', 'diff', '--name-only', 'HEAD']
    return _ChangedFiles(cmd)


def StagedFiles():
    cmd = ['git', 'diff', '--name-only', '--cached']
    return _ChangedFiles(cmd)


def UnstaedFiles():
    cmd = ['git', 'diff', '--name-only']
    return _ChangedFiles(cmd)


def Handle(commit_option, version_file_path):
    version_str_updated = version_handler.Handle(
        commit_option['version_index'], version_file_path)

    msg_tags = [commit_option['commit_type'],
                version_str_updated, commit_option['jira_id']]
    commit_msg = _GenerateCommitMsg(msg_tags, commit_option['commit_msg'])
    logging.debug('commit msg: ' + commit_msg)

    _Add(version_file_path)
    _Commit(commit_msg)

    print('commit: ' + commit_msg)


# if __name__ == "__main__":
    # _Add('test.py')
    # print(IsFileChanged('version.properties'))
    # _Commit('ABCDQWE')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='imit',
    version="0.0.10",
    description="提供交互或命令行的形式规范地提交git commit",
    long_description="""提供交互或命令行的形式规范地提交git commit""",
    keywords='python commit git',
    author='leytou',
    author_email='hi_litao@163.com',
    url='https://github.com/leytou',
    license='MIT',
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.7",
    install_requires=['inquirer', 'docopt', 'GitPython', 'jira', 'pyDes'],
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
    entry_points={'console_scripts': [
        'imit = src.imit:main',
    ]},
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import sys
import os
import inquirer


sys.path.append(os.path.dirname(os.path.realpath(__file__)))  # noqa
import version_handler
import jira_issues


def _MsgValidation(answers, current):
    if not current:
        raise inquirer.errors.ValidationError(
            '', reason='Please input commit message!')

    return True


def QType(types):
    question = inquirer.List('commit_type',
                             message='请选择commit类型',
                             choices=types,
                             carousel=True
                             )
    return inquirer.prompt([question])


def QVersion(tags, commit_type, version_file_path):
    tags_index_tagged = []
    nums = list(version_handler.TagNumDict(version_file_path).values())
    index = 0
    for tag in tags:
        tag = '(%d) ' % nums[index] + tag
        index += 1
        tags_index_tagged.append((tag, index))

    current_version = version_handler.CurrentVersion(version_file_path)

    if commit_type == 'modify' or commit_type == 'bugfix':
        index = -1
    elif commit_type == 'feature':
        index = -2
    question = inquirer.List('version_index',
                             message='请选择要增加的版本号(当前: %s)' % current_version,
                             choices=tags_index_tagged,
                             carousel=True,
                             default=tags_index_tagged[index][1]
                             )
    return inquirer.prompt([question])


def QJiraId(from_server=False):
    print('正在加载jira 问题列表...')
    issues = []
    if from_server:
        issues = jira_issues.GetJiraIssuesFromServer()
    else:
        issues = jira_issues.GetJiraIssues()
    if not issues:
        print('加载jira 问题列表失败...')
        return {'jira_id': ''}

    jira_ids = [(i[0] + ' ' + i[1], i[0]) for i in issues]
    jira_ids.insert(0,  ('无', ''))
    jira_ids.insert(4, ('↺【 刷新 】', '--refresh--'))
    question = inquirer.List('jira_id',
                             message='请选择JIRA ID',
                             choices=jira_ids,
                             carousel=True
                             )
    answer = inquirer.prompt([question])
    if answer['jira_id'] == '--refresh--':
        return QJiraId(True)
    else:
        return answer


def QMsg():
    question = inquirer.Text('commit_msg',
                             message='请输入commit message',
                             validate=_MsgValidation
                             )
    return inquirer.prompt([question])


def QUsernamePassword():
    question = [
        inquirer.Text(
            'username', message='请输入Jenkines用户名', validate=lambda _, x: x != ''),
        inquirer.Password('password', message='请输入Jenkines密码',
                          validate=lambda _, x: x != ''),
    ]
    return inquirer.prompt(question)




import configparser
import ast

from pathlib import Path

config_path = str(Path.home()) + '/.imitrc.ini'
group = 'user'


def Write(key, value):
    config = configparser.ConfigParser()
    config.read(config_path, encoding="utf-8")

    config[group][key] = value
    with open(config_path, 'w') as configfile:
        config.write(configfile)


def Get(key):
    config = configparser.ConfigParser()
    config.read(config_path, encoding="utf-8")
    if config.has_option(group, key):
        return config.get(group, key)
    return ''
